﻿using UniversityContracts.BindingModels;
using UniversityContracts.SearchModels;
using UniversityContracts.StoragesContracts;
using UniversityContracts.ViewModels;
using UniversityDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace UniversityDatabaseImplement.Implements
{
    internal class StatementStorage : IStatementStorage
    {
        public List<StatementViewModel> GetFullList()
        {
            using var context = new UniversityDatabase();
            return context.Statements
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public List<StatementViewModel> GetFilteredList(StatementSearchModel model)
        {
            using var context = new UniversityDatabase();
            return context.Statements
                .Where(x => (
                        (!model.Id.HasValue || x.Id == model.Id)
                    )
                )
                .Select(x => x.GetViewModel)
                .ToList();
        }
        public StatementViewModel? GetElement(StatementSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            return context.Statements
                .FirstOrDefault(x => (model.Id.HasValue && x.Id == model.Id))?.GetViewModel;
        }
        public StatementViewModel? Insert(StatementBindingModel model)
        {
            var newStatement = Statement.Create(model);
            if (newStatement == null)
            {
                return null;
            }
            using var context = new UniversityDatabase();
            context.Statements.Add(newStatement);
            context.SaveChanges();
            return newStatement.GetViewModel;
        }

        public StatementViewModel? Update(StatementBindingModel model)
        {
            using var context = new UniversityDatabase();
            var statement = context.Statements.FirstOrDefault(x => x.Id == model.Id);
            if (statement == null)
            {
                return null;
            }
            statement.Update(model);
            context.SaveChanges();
            return statement.GetViewModel;
        }
        public StatementViewModel? Delete(StatementBindingModel model)
        {
            using var context = new UniversityDatabase();
            var element = context.Statements.FirstOrDefault(rec => rec.Id == model.Id);
            if (element != null)
            {
                context.Statements.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
