﻿using UniversityContracts.BindingModels;
using UniversityContracts.ViewModels;
using UniversityDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;
using UniversityDataModels.Enums;

namespace UniversityDatabaseImplement.Models
{
    public class ExaminationResult : IExaminationResultModel
    {
        public int Id { get; set; }
        public string ExaminationForm { get; set; } = string.Empty;
        [Required]
        public MarkType Mark { get; set; }
        [Required]
        public DateTime Date { get; set; }
        [ForeignKey("ExaminationResultId")]
        public virtual List<Activity> Activities { get; set; } = new();
        [ForeignKey("ExaminationResultId")]
        public virtual List<StudentExaminationResult> StudentExaminationResults { get; set; } = new();
        public static ExaminationResult Create(ExaminationResultBindingModel model)
        {
            return new ExaminationResult
            {
                Id = model.Id,
                ExaminationForm = model.ExaminationForm,
                Mark = model.Mark,
                Date = model.Date
            };
        }

        public void Update(ExaminationResultBindingModel model)
        {
            if (model == null) return;

            Id = model.Id;
            ExaminationForm = model.ExaminationForm;
            Mark = model.Mark;
            Date = model.Date;
        }

        public ExaminationResultViewModel GetViewModel => new()
        {
            Id = Id,
            ExaminationForm = ExaminationForm,
            Mark = Mark,
            Date = Date
        };
    }
}
