﻿using UniversityContracts.BindingModels;
using UniversityContracts.ViewModels;
using UniversityDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace UniversityDatabaseImplement.Models
{
    public class Student : IStudentModel
    {
        public int Id { get; set; }
        [Required]
        public string Name { get; set; } = string.Empty;
        [Required]
        public string RecordCardNumber { get; set; } = string.Empty;
        [ForeignKey("StudentId")]
        public virtual List<StatementStudent> StatementStudents { get; set; } = new();
        [ForeignKey("StudentId")]
        public virtual List<StudentExaminationResult> StudentExaminationResults { get; set; } = new();

        private Dictionary<int, IExaminationResultModel>? _results;
        [NotMapped]
        public Dictionary<int, IExaminationResultModel> Results {
            get {
                if(_results == null)
                {
                    _results = StudentExaminationResults.ToDictionary(
                        x => x.ExaminationResult.Id, x => x.ExaminationResult as IExaminationResultModel);
                }

                return _results;
            }
        }

        private Dictionary<int, IStatementModel>? _statements;
        [NotMapped]
        public Dictionary<int, IStatementModel> Statements
        {
            get
            {
                if (_statements == null)
                {
                    _statements = StatementStudents.ToDictionary(
                        x => x.Statement.Id, x => x.Statement as IStatementModel);
                }

                return _statements;
            }
        }
        public static Student Create(StudentBindingModel model)
        {
            return new Student
            {
                Id = model.Id,
                Name = model.Name,
                RecordCardNumber = model.RecordCardNumber
            };
        }
        public void Update(StudentBindingModel model)
        {
            if (model == null) return;

            Id = model.Id;
            Name = model.Name;
            RecordCardNumber = model.RecordCardNumber;
        }

        public StudentViewModel GetViewModel => new()
        {
            Id = Id,
            Name = Name,
            RecordCardNumber = RecordCardNumber,
            Results = Results,
            Statements = Statements
        };

    }
}
